# postcreate script. You could setup a workflow here for example

try:
    from cubicweb import _
except ImportError:
    _ = unicode

# BlogEntry workflow
bwf = add_workflow(_("default BlogEntry workflow"), "BlogEntry")

draft = bwf.add_state(_("draft"), initial=True)
published = bwf.add_state(_("published"))

publish = bwf.add_transition(_("publish"), draft, published, ("managers", "owners"))



from cubicweb_web.views.urlrewrite import SimpleReqRewriter, rgx


class BlogReqRewriter(SimpleReqRewriter):
    rules = [
        # links generated by archives by date/author boxes
        (
            rgx(r"/(micro)?blog/([0-9]+)/blogentries"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B creation_date BD, "
                "B entry_of BL, BL eid %(eid)s" % {"eid": r"\2"},
            ),
        ),
        (
            rgx(r"/(micro)?blog/([0-9]+)/blogentries/([a-z_]+)"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B creation_date BD, "
                "B entry_of BL, BL eid %(eid)s, "
                'B created_by U, U login "%(user)s"' % {"eid": r"\2", "user": r"\3"},
                user=r"\2",
            ),
        ),
        (
            rgx(r"/(micro)?blog/([0-9]+)/blogentries/([0-9]{4})"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B creation_date BD, B entry_of BL, BL eid "
                "%(eid)s HAVING YEAR(BD)= %(year)s" % {"eid": r"\2", "year": r"\3"},
            ),
        ),
        (
            rgx(r"/(micro)?blog/([0-9]+)/blogentries/([0-9]{4})/([0-9]{2})"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B creation_date BD, "
                "B entry_of BL, BL eid %(eid)s "
                "HAVING YEAR(BD)= %(year)s, MONTH(BD)=%(month)s"
                % {"eid": r"\2", "year": r"\3", "month": r"\4"},
            ),
        ),
        # XXX use or kill
        (
            rgx(r"/blogentry/([a-z_]+)"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B is BlogEntry, "
                'B creation_date BD, B created_by U, U login "%(user)s"'
                % {"user": r"\1"},
                user=r"\1",
            ),
        ),
        (
            rgx(r"/blogentry/([a-z_]+)\.rss"),
            dict(
                rql="Any B,BD ORDERBY BD DESC LIMIT 20 WHERE B is BlogEntry, "
                'B creation_date BD, B created_by U, U login "%(user)s"'
                % {"user": r"\1"},
                vid="rss",
            ),
        ),
        (
            rgx(r"/blogentries/([0-9]{4})"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B is BlogEntry, B creation_date BD "
                "HAVING YEAR(BD) = %(year)s" % {"year": r"\1"}
            ),
        ),
        (
            rgx(r"/blogentries/([0-9]{4})/([0-9]{2})"),
            dict(
                rql="Any B,BD ORDERBY BD DESC WHERE B is BlogEntry, B creation_date BD "
                "HAVING YEAR(BD) = %(year)s, MONTH(BD) = %(month)s"
                % {"year": r"\1", "month": r"\2"}
            ),
        ),
    ]



"""Secondary views for blogs

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

import re

from calendar import monthrange
from datetime import datetime

from six import PY2
from six import text_type as unicode

from logilab.mtconverter import xml_escape

from cubicweb.schema import display_name
from cubicweb_web.view import EntityView
from cubicweb.predicates import paginated_rset, sorted_rset, is_instance
from cubicweb_web.views import uicfg
from cubicweb_web.views import primary, baseviews, calendar, navigation, workflow
from cubicweb_web.views.xmlrss import RSSItemView

try:
    from cubicweb import _
except ImportError:
    _ = unicode

__docformat__ = "restructuredtext en"

_pvs = uicfg.primaryview_section
_pvs.tag_attribute(("BlogEntry", "title"), "hidden")
_pvs.tag_attribute(("BlogEntry", "content"), "hidden")
_pvs.tag_subject_of(("BlogEntry", "entry_of", "*"), "relations")
_afs = uicfg.autoform_section
_afs.tag_subject_of(("BlogEntry", "entry_of", "Blog"), "main", "attributes")

# blog entries ###########################################################


def render_blogentry_title(req, w, entity):
    w("<h1>%s</h1>" % entity.view("incontext"))
    w('<div class="author_date"><div>%s' % req.format_date(entity.creation_date))
    rql = None
    if entity.has_creator:
        creator = entity.has_creator[0]
        name = creator.name
        rql = (
            "Any X ORDERBY D DESC WHERE X is BlogEntry, X has_creator Y, "
            "Y eid %s, X creation_date D" % creator.eid
        )
    elif entity.creator:
        creator = entity.creator
        name = creator.name()
        rql = (
            "Any X ORDERBY D DESC WHERE X is BlogEntry, X created_by Y, "
            "Y eid %s, X creation_date D" % creator.eid
        )
    if rql:
        vtitle = _("blog entries created by %s") % name
        url = req.build_url("view", rql=rql, vtitle=vtitle, page_size=10)
        w(
            ' %s <a title="%s" href="%s">%s</a>'
            % (_("by"), xml_escape(vtitle), xml_escape(url), xml_escape(name))
        )
    w("</div></div>")


class BlogEntryPrimaryView(primary.PrimaryView):
    __select__ = is_instance("BlogEntry")
    show_attr_label = False

    def render_entity_attributes(self, entity):
        super().render_entity_attributes(entity)
        # render the actual blog entry content outside the attributes table
        # which causes major CSS headaches, see bug #5450612.
        self.w('<div class="blogwrapper">')
        self.w(entity.printable_value("content"))
        self.w("</div>")

    def render_entity_title(self, entity):
        self._cw.add_css("cubes.blog.css")
        w = self.w
        w('<div class="blogentry_title">')
        render_blogentry_title(self._cw, w, entity)
        w("</div>")
        w('<br class="clear"/>')


# don't show workflow history for blog entry
class BlogEntryWFHistoryVComponent(workflow.WFHistoryVComponent):
    __select__ = workflow.WFHistoryVComponent.__select__ & is_instance("BlogEntry")

    def render(self, w, **kwargs):
        pass


class BlogEntrySameETypeListView(baseviews.SameETypeListView):
    __select__ = baseviews.SameETypeListView.__select__ & is_instance("BlogEntry")
    countrql = (
        "Any COUNT(B) WHERE B is BlogEntry, "
        "B creation_date >= %(firstday)s, B creation_date <= %(lastday)s"
    )
    item_vid = "blog"

    def call(self, **kwargs):
        self._cw.add_css("cubes.blog.css")
        super().call(**kwargs)
        # XXX Iirk, IPrevNext
        if "year" in self._cw.form and "month" in self._cw.form:
            self.render_next_previous(
                int(self._cw.form["year"]), int(self._cw.form["month"])
            )

    def render_next_previous(self, year, month):
        if month == 12:
            nextmonth = 1
            year = year + 1
        else:
            nextmonth = month + 1
        if month == 1:
            previousmonth = 12
            year = year - 1
        else:
            previousmonth = month - 1
        self.w('<div class="prevnext">')
        self.w(
            '<span class="previousmonth">%s</span>'
            % self.render_link(
                year, previousmonth, xml_escape("<< " + self._cw._("previous month"))
            )
        )
        self.w(
            '<span class="nextmonth">%s</span>'
            % self.render_link(
                year, nextmonth, xml_escape(self._cw._("next month") + " >>")
            )
        )
        self.w("</div>")

    def render_link(self, year, month, atitle):
        firstday = datetime(year, month, 1)
        lastday = datetime(year, month, monthrange(year, month)[1])
        args = {"firstday": firstday, "lastday": lastday}
        nmb_entries = self._cw.execute(self.countrql, args)[0][0]
        if not nmb_entries:
            return
        rql = (
            "Any B, BD ORDERBY BD DESC "
            "WHERE B is BlogEntry, B creation_date BD, "
            'B creation_date >= "%s", B creation_date <= "%s"'
            % (firstday.strftime("%Y-%m-%d"), lastday.strftime("%Y-%m-%d"))
        )
        label = "{} {} [{}]".format(
            self._cw._(calendar.MONTHNAMES[month - 1]),
            year,
            nmb_entries,
        )
        vtitle = "{} {}".format(display_name(self._cw, "BlogEntry", "plural"), label)
        url = self._cw.build_url("view", rql=rql, vtitle=vtitle, month=month, year=year)
        return f'<a href="{xml_escape(url)}">{atitle}</a>'


class BlogEntryBlogView(EntityView):
    __regid__ = "blog"
    __select__ = is_instance("BlogEntry")

    toolbar_components = (
        primary.PrimaryView.content_navigation_components.im_func
        if PY2
        else primary.PrimaryView.content_navigation_components
    )

    def cell_call(self, row, col, **kwargs):
        entity = self.cw_rset.get_entity(row, col)
        w = self.w
        w('<div class="post">')
        self.toolbar_components("ctxtoolbar")
        render_blogentry_title(self._cw, w, entity)
        w('<div class="entry">')
        body = entity.printable_value("content")
        w(body)
        w("</div>")
        w('<br class="clear"/>')
        w('<div class="postmetadata">%s</div>' % entity.view("post-reldata"))
        w("</div>")


class BlogEntryPostMetaData(EntityView):
    __regid__ = "post-reldata"
    __select__ = is_instance("BlogEntry")

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        _ = lambda ertype, form="": display_name(self._cw, ertype, form)
        reldata = []
        w = reldata.append
        schema = self._cw.vreg.schema
        if "comments" in schema and "BlogEntry" in schema.rschema("comments").objects():
            from cubicweb_comment.entities import subcomments_count

            count = subcomments_count(entity)
            if count:
                url = xml_escape(entity.absolute_url())
                if count > 1:
                    label = _("Comment", "plural")
                else:
                    label = _("Comment")
                w(f'<a href="{url}">{count} {label}</a>')
            else:
                w("{} {}".format(count, _("Comment")))
        if "tags" in schema and "BlogEntry" in schema.rschema("tags").objects():
            tag_rset = entity.related("tags", "object")
            if tag_rset:
                w("{} {}".format(_("tags", "object"), self._cw.view("csv", tag_rset)))
        rset = entity.related("entry_of", "subject")
        if rset:
            w(
                "{} {}".format(
                    self._cw._("blogged in "), self._cw.view("csv", rset, "null")
                )
            )
        self.w(" | ".join(reldata))


class BlogNavigation(navigation.PageNavigation):
    __select__ = paginated_rset() & sorted_rset() & is_instance("BlogEntry")

    def index_display(self, start, stop):
        return unicode(int(start / self.page_size) + 1)


# micro blog entries #####################################################


def format_microblog(entity):
    if entity.has_creator:
        author = entity.has_creator[0]
        if author.has_avatar:
            imgurl = author.has_avatar[0].uri
            ablock = '<a href="{}"><img src="{}" alt="avatar"/></a>'.format(
                author.absolute_url(),
                xml_escape(imgurl),
            )
        else:
            ablock = entity.has_creator[0].view("outofcontext")
    else:
        ablock = entity.dc_creator()
    if entity.content_format == "text/html":
        content = entity.content
    else:
        words = []
        for word in entity.content.split():
            if word.startswith("http://"):
                word = f'<a href="{word}">{word}</a>'
            else:
                word = xml_escape(word)
            words.append(word)
        content = " ".join(words)
    return (
        '<div class="microblog">'
        '<span class="author">%s</span>'
        '<span class="msgtxt">%s</span>'
        '<span class="meta"><a href="%s">%s</a></span>'
        "</div>" % (ablock, content, entity.absolute_url(), entity.creation_date)
    )


class MicroBlogEntryPrimaryView(primary.PrimaryView):
    __select__ = primary.PrimaryView.__select__ & is_instance("MicroBlogEntry")

    def cell_call(self, row, col):
        self._cw.add_css("cubes.blog.css")
        entity = self.cw_rset.get_entity(row, col)
        self.w(format_microblog(entity))


class MicroBlogEntrySameETypeListView(baseviews.SameETypeListView):
    __select__ = baseviews.SameETypeListView.__select__ & is_instance("MicroBlogEntry")

    def cell_call(self, row, col):
        self._cw.add_css("cubes.blog.css")
        entity = self.cw_rset.get_entity(row, col)
        self.w(format_microblog(entity))


_CLEAN_STYLE_RE = re.compile(r"<style .*?</style>", re.MULTILINE | re.DOTALL)


class BlogEntryRSSItemView(RSSItemView):
    __select__ = is_instance("BlogEntry")

    def render_description(self, entity):
        """
        Make sure generated XML for RSS is "valid", ie. do not
        contains <style> tags

        Note that this *should* not happen (style tags there), but
        browser tolerate them.
        """
        htmlcontent = entity.dc_description(format="text/html")
        htmlcontent = _CLEAN_STYLE_RE.sub("", htmlcontent)
        self._marker("description", htmlcontent)



# pylint: disable-msg=W0622
"""cubicweb-blog packaging information"""

modname = "blog"
distname = "cubicweb-%s" % modname

numversion = (2, 1, 0)
version = ".".join(str(num) for num in numversion)

license = "LGPL"
description = "blogging component for the CubicWeb framework"
web = "https://forge.extranet.logilab.fr/cubicweb/cubes/%s" % distname
mailinglist = "mailto://cubicweb@lists.logilab.org"
author = "Logilab"
author_email = "contact@logilab.fr"
classifiers = [
    "Environment :: Web Environment",
    "Framework :: CubicWeb",
    "Programming Language :: Python",
    "Programming Language :: JavaScript",
]

__depends__ = {
    "cubicweb": ">= 3.38.0, < 3.39.0",
    "cubicweb-sioc": ">= 0.2.2",
    "six": ">= 1.4.0",
}
__recommends__ = {
    "cubicweb-tag": None,
    "cubicweb-preview": None,
    "cubicweb-comment": ">= 1.6.3",
    "cubicweb-seo": None,
    "feedparser": None,
    "rdflib": None,
}



# XML <-> yams equivalence
from cubicweb.xy import xy

xy.add_equivalence("Blog", "sioc:Weblog")
xy.add_equivalence("BlogEntry", "sioc:BlogPost")
xy.add_equivalence("BlogEntry title", "dcterms:title")
xy.add_equivalence("BlogEntry content", "sioc:content")



from six import text_type as unicode

try:
    from cubicweb import _
except ImportError:
    _ = unicode

from yams.buildobjs import (
    EntityType,
    String,
    RichString,
    SubjectRelation,
    RelationDefinition,
)
from cubicweb.schema import WorkflowableEntityType, ERQLExpression


class Blog(EntityType):
    title = String(maxsize=50, required=True)
    description = RichString()
    rss_url = String(
        maxsize=128,
        description=_(
            "blog's rss url (useful for when using external site such as feedburner)"
        ),
    )


class BlogEntry(WorkflowableEntityType):
    __permissions__ = {
        "read": (
            "managers",
            "users",
            ERQLExpression('X in_state S, S name "published"'),
        ),
        "add": ("managers", "users"),
        "update": ("managers", "owners"),
        "delete": ("managers", "owners"),
    }
    title = String(required=True, fulltextindexed=True, maxsize=256)
    content = RichString(required=True, fulltextindexed=True)
    entry_of = SubjectRelation("Blog")
    same_as = SubjectRelation("ExternalUri")


class MicroBlog(EntityType):
    title = String(maxsize=50, required=True)
    description = RichString()


class MicroBlogEntry(EntityType):
    __permissions__ = {
        "read": ("managers", "users"),
        "add": ("managers", "users"),
        "update": ("managers", "owners"),
        "delete": ("managers", "owners"),
    }
    content = RichString(required=True, fulltextindexed=True)
    entry_of = SubjectRelation("MicroBlog")
    same_as = SubjectRelation("ExternalUri")


class UserAccount(EntityType):
    name = String(required=True)  # see foaf:accountName


class has_creator(RelationDefinition):
    subject = ("BlogEntry", "MicroBlogEntry")
    object = "UserAccount"


class has_avatar(RelationDefinition):
    subject = "UserAccount"
    object = "ExternalUri"



import unittest
from cubicweb.devtools.testlib import AutomaticWebTest


class AutomaticWebTest(AutomaticWebTest):
    def to_test_etypes(self):
        return {
            "Blog",
            "BlogEntry",
            "MicroBlog",
            "MicroBlogEntry",
        }


if __name__ == "__main__":
    unittest.main()



add_relation_definition("BlogEntry", "in_state", "State")
add_relation_definition("TrInfo", "wf_info_for", "BlogEntry")
add_relation_definition("BlogEntry", "custom_workflow", "Workflow")


# add BlogEntry workflow
if confirm("add blog entry workflow"):
    bwf = add_workflow(_("default BlogEntry workflow"), "BlogEntry")

    draft = bwf.add_state(_("draft"), initial=True)
    published = bwf.add_state(_("published"))

    publish = bwf.add_transition(_("publish"), draft, published, ("managers",))
    commit()

# set state to published for already existing blog entries
blogentries = rql("Any B WHERE B is BlogEntry")

for (eid,) in blogentries:
    session.unsafe_execute(
        'SET B in_state S WHERE S name "published", '
        'S state_of WF, WF name "default BlogEntry workflow", '
        "B eid %(b)s",
        {"b": eid},
        "b",
    )

commit()



from datetime import datetime
from six import text_type as unicode

from lxml.html import fromstring, tostring

try:
    import feedparser
except ImportError:
    feedparser = None

try:
    import rdflib
except ImportError:
    rdflib = None
else:
    RDF = rdflib.Namespace("http://www.w3.org/1999/02/22-rdf-syntax-ns#")
    SIOC = rdflib.Namespace("http://rdfs.org/sioc/ns#")
    DCTERMS = rdflib.Namespace("http://purl.org/dc/terms/")

try:
    from cubicweb_datafeed.sobjects import DataFeedParser
except ImportError:
    DataFeedParser = None


def get_subject(g, pred, obj):
    subjects = list(g.subjects(pred, obj))
    assert len(subjects) == 1
    return subjects[0]


def get_object(g, subj, pred):
    objects = list(g.objects(subj, pred))
    assert len(objects) == 1
    return objects[0]


def parse_blogpost_sioc(url):
    g = rdflib.ConjunctiveGraph()
    g.parse(url)
    for post, type_, blogpost_ in g.triples((None, RDF.type, SIOC.BlogPost)):
        item = {"uri": unicode(post)}
        item["title"] = unicode(get_object(g, post, DCTERMS.title))
        item["content"] = unicode(get_object(g, post, SIOC.content))
        yield item


format_map = {
    "application/xhtml+xml": "text/html",
    "text/html": "text/html",
    "text/plain": "text/plain",
}

IMG_SPIES = [
    "http://feeds.feedburner.com",
    "http://creatives.commindo-media",
    "http://imp.constantcontact.com",
    "https://blogger.googleusercontent.com/tracker",
    "http://stats.wordpress.com/",
]


def is_img_spy(node):
    if node.tag != "img":
        return False
    for url in IMG_SPIES:
        if node.get("src").startswith(url):
            return True
    return False


def is_tweetmeme_spy(node):
    href = node.get("href")
    if href and href.startswith("http://api.tweetmeme.com/share"):
        return True
    return False


def remove_content_spies(content):
    root = fromstring(content)
    if is_img_spy(root):
        return ""
    for img in root.findall(".//img"):
        if is_img_spy(img):
            img.drop_tree()
        elif img.get("height") == "1" and img.get("width") == "1":
            print(tostring(img), "is probably a spy")
    for anchor in root.findall(".//a"):
        if is_tweetmeme_spy(anchor):
            anchor.drop_tree()
    return unicode(tostring(root))


def parse_blogpost_rss(url):
    data = feedparser.parse(url)
    feed = data.feed
    for entry in data.entries:
        item = {}
        if "feedburner_origlink" in entry:
            item["uri"] = entry.feedburner_origlink
        else:
            item["uri"] = entry.link
        item["title"] = entry.title
        if hasattr(entry, "content"):
            content = entry.content[0].value
            mimetype = entry.content[0].type
        elif hasattr(entry, "summary_detail"):
            content = entry.summary_detail.value
            mimetype = entry.summary_detail.type
        else:
            content = ""  # XXX entry.description?
            mimetype = "text/plain"
        if mimetype == "text/html":
            content = remove_content_spies(content)
        item["content"] = content
        item["content_format"] = format_map.get(mimetype, "text/plain")
        if hasattr(entry, "date_parsed"):
            item["creation_date"] = datetime(*entry.date_parsed[:6])
        if hasattr(entry, "author_detail") and hasattr(entry.author_detail, "href"):
            item["author"] = entry.author_detail.href
        elif hasattr(feed, "author_detail") and hasattr(feed.author_detail, "href"):
            item["author"] = feed.author_detail.href
        elif hasattr(feed, "author"):
            item["author"] = feed.author
        elif hasattr(feed, "image") and hasattr(feed.image, "link"):
            item["author"] = feed.image.link
        else:
            item["author"] = url
        item["cwuri"] = feed.link
        yield item


def parse_microblogpost_rss(url):
    feed = feedparser.parse(url)
    for entry in feed.entries:
        item = {}
        item["uri"] = entry.id
        # fix weird parsing
        if hasattr(entry, "content"):
            content = entry.content[0].value
            mimetype = entry.content[0].type
        else:
            content = entry.description
            mimetype = "text/plain"
        if ": " in content:
            author, text = content.split(": ", 1)
            if " " not in author:
                content = text
        item["content"] = content
        item["content_format"] = format_map.get(mimetype, "text/plain")
        item["creation_date"] = datetime(*entry.date_parsed[:6])
        item["modification_date"] = datetime(*entry.date_parsed[:6])
        item["author"] = feed.channel.link  # true for twitter
        item["cwuri"] = feed.channel.link
        for link in entry.links:
            if link.type.startswith("image/") and link.rel == "image":
                item["avatar"] = link.href
                break
        else:
            screen_name = feed.channel.link.split("/")[-1]
            item["avatar"] = get_twitter_avatar(screen_name)
        yield item


def search_twitter(word):
    import urllib2
    from simplejson import loads

    data = urllib2.urlopen(
        "http://search.twitter.com/search.json?q=%s&rpp=100" % word
    ).read()
    loads(data)
    # process results
    # print results
    return []


AVATAR_CACHE = {}


def get_twitter_avatar(screen_name):
    if screen_name not in AVATAR_CACHE:
        from urllib2 import urlopen
        import simplejson

        data = urlopen(
            "http://api.twitter.com/1/users/show.json?screen_name=%s" % screen_name
        ).read()
        user = simplejson.loads(data)
        AVATAR_CACHE[screen_name] = user["profile_image_url"]
    return AVATAR_CACHE[screen_name]


if DataFeedParser is not None:

    class BlogPostParser(DataFeedParser):
        __abstract__ = True
        entity_type = "BlogEntry"

        def process(self, url):
            stats = {"update": 0, "creation": 0}
            for item in self.parse(url):
                author = item.pop("author", None)
                avatar = item.pop("avatar", None)
                euri = self.sget_entity("ExternalUri", uri=item.pop("uri"))
                if euri.same_as:
                    # sys.stdout.write('.')
                    stats["update"] += 1
                    post = self.update_blogpost(euri.same_as[0], item)
                else:
                    # sys.stdout.write('+')
                    stats["creation"] += 1
                    post = self.create_blogpost(item, euri)
                if author:
                    account = self.sget_entity("UserAccount", name=author)
                    self.sget_relation(post.eid, "has_creator", account.eid)
                    if avatar:
                        auri = self.sget_entity("ExternalUri", uri=avatar)
                        self.sget_relation(account.eid, "has_avatar", auri.eid)
                # sys.stdout.flush()
            return stats

        def create_blogpost(self, item, uri):
            entity = self._cw.create_entity(self.entity_type, **item)
            entity.set_relations(same_as=uri)
            return entity

        def update_blogpost(self, entity, item):
            entity.cw_set(**item)
            return entity

    if rdflib is not None:

        class BlogPostSiocParser(BlogPostParser):
            __regid__ = "blogpost-sioc"
            parse = staticmethod(parse_blogpost_sioc)

    if feedparser is not None:

        class BlogPostRSSParser(BlogPostParser):
            __regid__ = "blogpost-rss"
            parse = staticmethod(parse_blogpost_rss)

        class MicroBlogPostRSSParser(BlogPostParser):
            __regid__ = "microblogpost-rss"
            entity_type = "MicroBlogEntry"
            parse = staticmethod(parse_microblogpost_rss)


if __name__ == "__main__":
    import sys
    from pprint import pprint

    name = sys.argv[1]
    url = sys.argv[2]

    parser = globals()[name]
    pprint(list(parser(url)))



"""Various blog boxes: archive, per author, etc...

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from six import text_type as unicode

from cubicweb import tags
from cubicweb.predicates import (
    none_rset,
    one_line_rset,
    is_instance,
    has_related_entities,
    match_view,
)
from cubicweb_web import component

try:
    from cubicweb import _
except ImportError:
    _ = unicode

__docformat__ = "restructuredtext en"


class BlogArchivesBox(component.CtxComponent):
    """blog side box displaying a Blog Archive"""

    __regid__ = "blog.archives_by_date"
    __select__ = (
        component.CtxComponent.__select__
        & is_instance("Blog", "MicroBlog")
        & has_related_entities("entry_of", "object")
    )
    title = _("blog.archives_by_date")
    order = 35
    context = "left"

    def render_body(self, w):
        # FIXME doesn't handle (yet) multiple blogs
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        rset = entity.related("entry_of", "object")
        self._cw.view(
            "cw.archive.by_date",
            rset,
            maxentries=6,
            basepath=entity.rest_path() + "/blogentries",
            w=w,
        )


class BlogEntryArchivesBox(BlogArchivesBox):
    __regid__ = "blog.entry_archives_by_date"
    __select__ = (
        component.CtxComponent.__select__
        & is_instance("BlogEntry", "MicroBlogEntry")
        & has_related_entities("entry_of", "subject")
    )

    def render_body(self, w):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        box = self._cw.vreg["ctxcomponents"].select(
            "blog.archives_by_date",
            self._cw,
            w=w,
            rset=entity.related("entry_of", "subject"),
        )
        box.render_body(w)


class BlogByAuthorBox(component.CtxComponent):
    __regid__ = "blog.archives_by_author"
    __select__ = (
        component.CtxComponent.__select__
        & is_instance("Blog", "MicroBlog")
        & has_related_entities("entry_of", "object")
    )
    title = _("blog.archives_by_author")
    order = 36
    context = "left"

    def render_body(self, w):
        # FIXME doesn't handle (yet) multiple blogs
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        rset = entity.related("entry_of", "object")
        self._cw.view(
            "cw.archive.by_author",
            rset,
            basepath=entity.rest_path() + "/blogentries",
            w=w,
        )


class BlogEntryByAuthorBox(BlogByAuthorBox):
    __regid__ = "blog.entry_archives_by_author"
    __select__ = (
        component.CtxComponent.__select__
        & is_instance("BlogEntry", "MicroBlogEntry")
        & has_related_entities("entry_of", "subject")
    )

    def render_body(self, w):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        box = self._cw.vreg["ctxcomponents"].select(
            "blog.archives_by_author",
            self._cw,
            w=w,
            rset=entity.related("entry_of", "subject"),
        )
        box.render_body(w)


class LatestBlogsBox(component.CtxComponent):
    """display a box with latest blogs and rss"""

    __regid__ = "blog.latest_blogs"
    __select__ = component.CtxComponent.__select__ & none_rset() & match_view("index")
    title = _("blog.latest_blogs")
    order = 34
    display_see_more_link = True
    contextual = False

    def latest_blogs_rset(self):
        return self._cw.execute(
            "Any X,T,CD ORDERBY CD DESC LIMIT 5 WHERE X is IN (MicroBlogEntry, BlogEntry), "
            "X title T, X creation_date CD"
        )

    def render_body(self, w):
        # XXX turn into a predicate
        rset = self.latest_blogs_rset()
        if not rset:
            return
        w('<ul class="boxListing">')
        for entity in rset.entities():
            w("<li>%s</li>\n" % tags.a(entity.dc_title(), href=entity.absolute_url()))
        rqlst = rset.syntax_tree()
        rqlst.set_limit(None)
        rql = rqlst.as_string(kwargs=rset.args)
        if self.display_see_more_link:
            url = self._cw.build_url("view", rql=rql, page_size=10)
            w("<li>%s</li>\n" % tags.a("[%s]" % self._cw._("see more"), href=url))
        rss_icon = self._cw.uiprops["RSS_LOGO_16"]
        # FIXME - could use rss_url defined as a property if available
        rss_label = '{} <img src="{}" alt="{}"/>'.format(
            self._cw._("subscribe"),
            rss_icon,
            self._cw._("rss icon"),
        )
        blogs = self._cw.execute("Any B,RSS WHERE B is Blog, B rss_url RSS")
        if len(blogs) == 1:
            rss_url = blogs[0][1]
        else:
            rss_url = self._cw.build_url("view", vid="rss", rql=rql)
        w("<li>%s</li>\n" % tags.a(rss_label, href=rss_url, escapecontent=False))
        w("</ul>\n")


class LatestBlogsBlogBox(LatestBlogsBox):
    """display a box with latest blogs and rss, filtered for a particular blog"""

    __select__ = (
        component.CtxComponent.__select__ & one_line_rset() & is_instance("Blog")
    )
    display_see_more_link = False
    contextual = True

    def latest_blogs_rset(self):
        blog = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        return self._cw.execute(
            "Any X,T,CD ORDERBY CD DESC LIMIT 5 WHERE "
            "X title T, X creation_date CD, X entry_of B, B eid %(b)s",
            {"b": blog.eid},
        )


class LatestBlogsBlogEntryBox(LatestBlogsBox):
    """display a box with latest blogs and rss, filtered for a particular blog"""

    __select__ = (
        component.CtxComponent.__select__
        & is_instance("BlogEntry")
        & has_related_entities("entry_of", "subject")
    )
    display_see_more_link = False
    contextual = True

    def latest_blogs_rset(self):
        blogentry = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        # FIXME doesn't handle (yet) multiple blogs
        blog = blogentry.related("entry_of", "subject").get_entity(0, 0)
        return self._cw.execute(
            "Any X,T,CD ORDERBY CD DESC LIMIT 5 WHERE "
            "X title T, X creation_date CD, X entry_of B, B eid %(b)s",
            {"b": blog.eid},
        )



"""Blog unit tests"""

import unittest
from cubicweb.devtools.testlib import CubicWebTC, MAILBOX


class BlogTestsCubicWebTC(CubicWebTC):
    """test blog specific behaviours"""

    def test_notifications(self):
        with self.admin_access.client_cnx() as cnx:
            cubicweb_blog = cnx.create_entity(
                "Blog", title="cubicweb", description="cubicweb c'est beau"
            )
            blog_entry_1 = cnx.create_entity(
                "BlogEntry", title="hop", content="cubicweb hop"
            )
            blog_entry_1.cw_set(entry_of=cubicweb_blog)
            blog_entry_2 = cnx.create_entity(
                "BlogEntry", title="yes", content="cubicweb yes"
            )
            blog_entry_2.cw_set(entry_of=cubicweb_blog)
            self.assertEqual(len(MAILBOX), 0)
            cnx.commit()
            self.assertEqual(len(MAILBOX), 0)
            blog_entry_1.cw_adapt_to("IWorkflowable").fire_transition("publish")
            cnx.commit()
            self.assertEqual(len(MAILBOX), 1)
            mail = MAILBOX[0]
            self.assertEqual(mail.subject, "[data] hop")
            blog_entry_2.cw_adapt_to("IWorkflowable").fire_transition("publish")
            cnx.commit()
            self.assertEqual(len(MAILBOX), 2)
            mail = MAILBOX[1]
            self.assertEqual(mail.subject, "[data] yes")

    def test_rss(self):
        with self.admin_access.client_cnx() as cnx:
            cubicweb_blog = cnx.create_entity(
                "Blog", title="cubicweb", description="cubicweb c'est beau"
            )
            content = """
<style> toto </style> tutu <style class=macin> toto
</style>
tutu
tutu
<style></style>

<tag>tutu</tag>

"""
            blog_entry_1 = cnx.create_entity(
                "BlogEntry", title="hop", content=content, content_format="text/html"
            )
            blog_entry_1.cw_set(entry_of=cubicweb_blog)

            xml = blog_entry_1.view("rssitem")
            self.assertEqual(xml.count("toto"), 0)
            self.assertEqual(xml.count("tutu"), content.count("tutu"))

    def test_prevnext(self):
        with self.admin_access.client_cnx() as cnx:
            e1 = cnx.create_entity("BlogEntry", title="a", content="a")
            cnx.create_entity("CWGroup", name="NotTheNextOfe1")
            e3 = cnx.create_entity("BlogEntry", title="b", content="b")
            cnx.commit()
            a1 = e1.cw_adapt_to("IPrevNext")
            self.assertIsNone(a1.previous_entity())
            self.assertEqual("b", a1.next_entity().title)
            a3 = e3.cw_adapt_to("IPrevNext")
            self.assertEqual("a", a3.previous_entity().title)
            self.assertIsNone(a3.next_entity())


if __name__ == "__main__":
    unittest.main()



"""Primary views for blogs

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""

from logilab.mtconverter import xml_escape

from cubicweb.utils import UStringIO
from cubicweb.predicates import is_instance
from cubicweb_web import component
from cubicweb_web.views import uicfg, primary

__docformat__ = "restructuredtext en"

_pvs = uicfg.primaryview_section
_pvdc = uicfg.primaryview_display_ctrl
_abaa = uicfg.actionbox_appearsin_addmenu
for etype in ("Blog", "MicroBlog"):
    _pvs.tag_attribute((etype, "title"), "hidden")
    _pvs.tag_object_of(("*", "entry_of", etype), "hidden")
    _pvdc.tag_attribute((etype, "description"), {"showlabel": False})
    _abaa.tag_object_of(("*", "entry_of", etype), True)
_pvs.tag_attribute(("Blog", "rss_url"), "hidden")

_pvs.tag_object_of(("*", "has_creator", "UserAccount"), "relations")
_pvs.tag_attribute(("UserAccount", "name"), "hidden")


class BlogPrimaryView(primary.PrimaryView):
    __select__ = is_instance("Blog", "MicroBlog")

    def entity_call(self, entity):
        self.w('<div class="blogwrapper">')
        super().entity_call(entity)
        self.w("</div>")

    def render_entity_relations(self, entity):
        super().render_entity_relations(entity)
        rset = entity.related("entry_of", "object")
        if rset:
            strio = UStringIO()
            self.paginate(w=strio.write, page_size=10, rset=rset)
            self.w(strio.getvalue())
            self.wview("sameetypelist", rset, showtitle=False)
            self.w(strio.getvalue())


class SubscribeToBlogComponent(component.EntityCtxComponent):
    __regid__ = "blogsubscribe"
    __select__ = component.EntityVComponent.__select__ & is_instance(
        "Blog", "MicroBlog"
    )
    context = "ctxtoolbar"

    def render_body(self, w):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        w(
            '<a href="%s"><img src="%s" alt="%s"/></a>'
            % (
                xml_escape(entity.cw_adapt_to("IFeed").rss_feed_url()),
                self._cw.uiprops["RSS_LOGO_16"],
                self._cw._("subscribe to this blog"),
            )
        )



"""entity classes for Blog entities

:organization: Logilab
:copyright: 2003-2010 LOGILAB S.A. (Paris, FRANCE)
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
:license: Lesser General Public License version 2 or above - http://www.gnu.org/
"""
# __docformat__ = "restructuredtext en"

from logilab.common.date import todate

from rdflib import URIRef, Literal, Namespace

from cubicweb.entities import AnyEntity, fetch_config
from cubicweb.entities.adapters import (
    EntityRDFAdapter,
    CWUserRDFAdapter as _CWUserRDFAdapter,
)
from cubicweb.entity import EntityAdapter
from cubicweb.predicates import is_instance
from cubicweb.rdf import NAMESPACES

NAMESPACES["sioc"] = Namespace("http://rdfs.org/sioc/ns#")
NAMESPACES["sioct"] = Namespace("http://rdfs.org/sioc/types#")
NAMESPACES["dcterms"] = Namespace("http://purl.org/dc/terms/")

__docformat__ = "restructuredtext en"


class CWUserRDFAdapter(_CWUserRDFAdapter):
    def triples(self):
        yield from super().triples()
        SIOC = self._use_namespace("sioc")
        uri = URIRef(self.uri)
        blog_entries = self._cw.find("BlogEntry", created_by=self.entity)
        for entry in blog_entries.entities():
            yield (uri, SIOC.creator_of, URIRef(entry.cwuri))


class BlogRDFAdapter(EntityRDFAdapter):
    __regid__ = "rdf"
    __select__ = is_instance("Blog")

    def triples(self):
        SIOC = self._use_namespace("sioc")
        SIOCT = self._use_namespace("sioct")
        RDF = self._use_namespace("rdf")
        DCTERMS = self._use_namespace("dcterms")
        uri = URIRef(self.uri)
        yield (uri, RDF.type, SIOCT.Weblog)
        yield (uri, DCTERMS.title, Literal(self.entity.dc_title()))
        if self.entity.description:
            yield (uri, DCTERMS.description, Literal(self.entity.description))
        yield (uri, SIOC.num_items, Literal(len(self.entity.reverse_entry_of)))
        yield (uri, SIOC.feed, URIRef(self.entity.cw_adapt_to("IFeed").rss_feed_url()))

        if self.entity.creator:
            yield (uri, SIOC.has_creator, URIRef(self.entity.creator.cwuri))

        for blog_entry in self.entity.reverse_entry_of:
            yield (uri, SIOC.container_of, URIRef(blog_entry.cwuri))


class BlogEntryRDFAdapter(EntityRDFAdapter):
    __regid__ = "rdf"
    __select__ = is_instance("BlogEntry")

    def triples(self):
        yield from self.sioc_triples()
        yield from self.schema_triples()

    def sioc_triples(self):
        SIOC = self._use_namespace("sioc")
        SIOCT = self._use_namespace("sioct")
        RDF = self._use_namespace("rdf")
        DCTERMS = self._use_namespace("dcterms")
        uri = URIRef(self.uri)
        yield (uri, RDF.type, SIOCT.BlogPost)
        yield (uri, DCTERMS.title, Literal(self.entity.dc_title()))
        yield (uri, DCTERMS.date, Literal(self.entity.creation_date))
        yield (uri, DCTERMS.modified, Literal(self.entity.modification_date))
        yield (uri, SIOC.content, Literal(self.entity.content))
        yield (uri, DCTERMS["format"], Literal(self.entity.content_format))
        if self.entity.entry_of:
            yield (uri, SIOC.container, URIRef(self.entity.entry_of[0].cwuri))
        if self.entity.creator:
            yield (uri, SIOC.has_owner, URIRef(self.entity.creator.cwuri))

    def schema_triples(self):
        RDF = self._use_namespace("rdf")
        SCHEMA = self._use_namespace("schema")
        uri = URIRef(self.uri)
        yield (uri, RDF.type, SCHEMA.BlogPosting)
        yield (uri, SCHEMA.headline, Literal(self.entity.dc_title()))
        yield (uri, SCHEMA.dateCreated, Literal(self.entity.creation_date))
        yield (uri, SCHEMA.dateModified, Literal(self.entity.modification_date))
        yield (uri, SCHEMA.text, Literal(self.entity.content))
        # yield (uri, DCTERMS["format"], Literal(self.entity.content_format))
        if self.entity.entry_of:
            yield (URIRef(self.entity.entry_of[0].cwuri), SCHEMA.blogPost, uri)
        if self.entity.creator:
            yield (uri, SCHEMA.creator, URIRef(self.entity.creator.cwuri))


class MicroBlogRDFAdapter(BlogRDFAdapter):
    __regid__ = "rdf"
    __select__ = is_instance("MicroBlog")

    def triples(self):
        yield from super().triples()
        SIOCT = self._use_namespace("sioct")
        RDF = self._use_namespace("rdf")
        uri = URIRef(self.uri)
        yield (uri, RDF.type, SIOCT.Microblog)


class MicroBlogEntryRDFAdapter(BlogEntryRDFAdapter):
    __regid__ = "rdf"
    __select__ = is_instance("MicroBlogEntry")

    def triples(self):
        yield from super().triples()
        SIOCT = self._use_namespace("sioct")
        RDF = self._use_namespace("rdf")
        uri = URIRef(self.uri)
        yield (uri, RDF.type, SIOCT.MicroblogPost)


###################################################


class BlogIFeedAdapter(EntityAdapter):
    __regid__ = "IFeed"
    __select__ = is_instance("Blog", "MicroBlog")

    def rss_feed_url(self):
        if getattr(self.entity, "rss_url", None):
            return self.entity.rss_url
        rql = "Any E ORDERBY D DESC " "WHERE E entry_of X, X eid %s, E creation_date D"
        return self._cw.build_url(
            rql=rql % self.entity.eid, vid="rss", vtitle=self.entity.dc_title()
        )


class BlogISiocContainerAdapter(EntityAdapter):
    __regid__ = "ISIOCContainer"
    __select__ = is_instance("Blog")

    def isioc_type(self):
        return "Weblog"

    def isioc_items(self):
        return self.entity.reverse_entry_of


class BlogEntry(AnyEntity):
    """customized class for BlogEntry entities"""

    __regid__ = "BlogEntry"
    fetch_attrs, cw_fetch_order = fetch_config(["creation_date", "title"], order="DESC")

    def dc_title(self):
        return self.title

    def dc_description(self, format="text/plain"):
        return self.printable_value("content", format=format)

    def dc_date(self, date_format=None):
        dc_date = self.creation_date
        for tr_info in self.reverse_wf_info_for:
            if tr_info.new_state.name == "published":
                dc_date = tr_info.creation_date
                break
        return self._cw.format_date(dc_date, date_format=date_format)


class BlogEntryICalendarableAdapter(EntityAdapter):
    __regid__ = "ICalendarable"
    __select__ = is_instance("BlogEntry")

    @property
    def start(self):
        return self.entity.creation_date

    @property
    def stop(self):
        return self.entity.creation_date


class BlogEntryICalendarViewsAdapter(EntityAdapter):
    __regid__ = "ICalendarViews"
    __select__ = is_instance("BlogEntry")

    def matching_dates(self, begin, end):
        """calendar views interface"""
        mydate = self.entity.creation_date
        if not mydate:
            return []
        mydate = todate(mydate)
        if begin < mydate < end:
            return [mydate]
        return []


class BlogEntryISiocItemAdapter(EntityAdapter):
    __regid__ = "ISIOCItem"
    __select__ = is_instance("BlogEntry")

    def isioc_content(self):
        return self.entity.content

    def isioc_container(self):
        return self.entity.entry_of and self.entity.entry_of[0] or None

    def isioc_type(self):
        return "BlogPost"

    def isioc_replies(self):
        # XXX link to comments
        return []

    def isioc_topics(self):
        # XXX link to tags, folders?
        return []


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (CWUserRDFAdapter,))
    vreg.register_and_replace(CWUserRDFAdapter, _CWUserRDFAdapter)



from cubicweb.predicates import is_instance
from cubicweb.entity import EntityAdapter
from cubicweb_web.views import ibreadcrumbs
from cubicweb_web.views.autoform import AutomaticEntityForm


class BlogEntryIBreadCrumbsAdapter(ibreadcrumbs.IBreadCrumbsAdapter):
    __select__ = is_instance("BlogEntry", "MicroBlogEntry")

    def parent_entity(self):
        return self.entity.entry_of and self.entity.entry_of[0] or None


class BlogEntryIPrevNextAdapter(EntityAdapter):
    __regid__ = "IPrevNext"
    __select__ = is_instance("BlogEntry", "MicroBlogEntry")

    def next_entity(self):
        return self._sibling_entry("ASC", ">")

    def previous_entity(self):
        return self._sibling_entry("DESC", "<")

    def _sibling_entry(self, order, operator):
        if self.entity.entry_of:
            rql = (
                "Any B ORDERBY B %s LIMIT 1 "
                "WHERE B entry_of BL, BL eid %%(blog)s, "
                "B eid %s %%(eid)s"
            )
            rset = self._cw.execute(
                rql % (order, operator),
                {"blog": self.entity.entry_of[0].eid, "eid": self.entity.eid},
            )
        else:
            rql = (
                "Any B ORDERBY B %s LIMIT 1 "
                "WHERE B eid %s %%(eid)s, NOT B entry_of BL, "
                'B is ET, ET name IN ("BlogEntry", "MicroBlogEntry")'
            )
            rset = self._cw.execute(rql % (order, operator), {"eid": self.entity.eid})
        if rset:
            return rset.get_entity(0, 0)


def registration_callback(vreg):
    vreg.register(BlogEntryIBreadCrumbsAdapter)
    vreg.register(BlogEntryIPrevNextAdapter)

    loaded_cubes = vreg.config.cubes()

    if "seo" in loaded_cubes:
        from cubicweb_seo.views import SitemapRule

        class BlogEntrySitemapRule(SitemapRule):
            __regid__ = "blogentry"
            query = "Any X WHERE X is BlogEntry"
            priority = 1.0
            chfreq = "yearly"

        class MicroBlogEntrySitemapRule(SitemapRule):
            __regid__ = "microblogentry"
            query = "Any X WHERE X is MicroBlogEntry"
            priority = 1.0
            chfreq = "yearly"

        vreg.register(BlogEntrySitemapRule)
        vreg.register(MicroBlogEntrySitemapRule)

    if "preview" in loaded_cubes:
        from cubicweb_preview.views.forms import PreviewFormMixin

        class PreviewAutomaticEntityForm(PreviewFormMixin, AutomaticEntityForm):
            preview_mode = "inline"
            __select__ = AutomaticEntityForm.__select__ & is_instance(
                "Blog", "BlogEntry", "MicroBlog", "MicroBlogEntry"
            )

        vreg.register(PreviewAutomaticEntityForm)



from cubicweb.predicates import is_instance, is_in_state
from cubicweb.sobjects.notification import NotificationView, StatusChangeMixIn


class BlogEntryPublishedView(StatusChangeMixIn, NotificationView):
    """get notified from published blogs"""

    __select__ = is_instance(
        "BlogEntry",
    ) & is_in_state("published")
    content_attr = "content"

    def subject(self):
        entity = self.cw_rset.get_entity(0, 0)
        return f"[{self._cw.vreg.config.appid}] {entity.dc_title()}"
